function [E,E_index,seed] = EIA_NABO(X,p,varargin)
% Written by Ruben Marrero <ruben.marrero-gomez@obs.ujf-grenoble.fr>
% Institut de Planetologie et d'Astrophysique de Grenoble (IPAG)
% http://ipag.osug.fr
%
% Modified by Miguel Angel Veganzones <miguel-angel.veganzones@gipsa-lab.fr>
% Grenoble Image Parole Signal Analysis laboratory (GIPSA-lab)
% http://www.gipsa-lab.grenoble-inp.fr
%
% Negative ABundance-Oriented (NABO) endmembers induction algorithm
% ------------------------------------------------------------------------------
% Input:            X : input data matrix [nvariables x nsamples]
%                   p: number of endmembers to induce (p > 0).
%
% Optional input:   {'p_init',value > 1} : initial number of endmembers (default 3).
%                   The algorithm starts to induce endmembers from a
%                   subspace of dimension p_init-1. A small value of p_init
%                   will improve the convergence to an optimal solution
%                   while a value close to p will reduce the computation
%                   cost.
%                   {'p_search',true|false(default)} : indicator to perform an automatic estimation of p.
%                   If p_search == true, the algorithm will automatically search for the optimal 
%                   number of endmembers in the range (p_init,p).
%                   {'iterations',value > 0} : indicates the number of samples
%                   that will be checked in order to find a replacement of
%                   the current endmembers (default 1).
%                   {'seed',value} : indicates the indexes of the data samples
%                   that will be selected as seeds to the algorithm (default []).
%                   Otherwise, the algorithm will select the seeds
%                   randomly. The value of 'seed' parameter is an array
%                   with p indexes in the range (1,nsamples).
%
% Output:           E : set of induced endmembers [nvariables x p]
%                   E_index : indexes of the induced endmembers [1 x p]
%                   seed : indexes of the seed samples [1 x p]
%
% Bibliographical references:
% [1] Marrero, R.; Lopez, S.; Callico, G.M.; Veganzones, M.A.; Plaza, A.; Chanussot, J.; Sarmiento, R., 
%     "A Novel Negative Abundance‐Oriented Hyperspectral Unmixing Algorithm," 
%     Geoscience and Remote Sensing, IEEE Transactions on , vol.53, no.7, pp.3772,3790, July 2015.

%% Parameters

% Checking
narginchk(2,10);
% Number of pixels
N = size(X, 2);
% defaults
p_init = 3;
p_search = false;
iterations = 1;
seed = [];
% additional inputs
nVarargs = length(varargin);
fprintf('Inputs in varargin(%d):\n',nVarargs)
for k = 1:2:nVarargs
    if strcmpi(varargin{k},'p_init')
        p_init = varargin{k+1};
        if p_init < 2
            p_init = 3;
            warning('p_init value smaller than 2. Set to default value (p_init=3)');
        end
    elseif strcmpi(varargin{k},'p_search')
        p_search = varargin{k+1};
    elseif strcmpi(varargin{k},'iterations')
        iterations = varargin{k+1};
        if iterations < 1
            iterations = 1;
            warning('iterations value smaller than 1. Set to default value (iterations=1)');
        end
    elseif strcmpi(varargin{k},'seed')
        seed = varargin{k+1};
        if length(seed) ~= p || min(seed) < 1 || max(seed) > N
            seed = [];
            warning('Incorrect seed. Set to default value (seed=[])');
        end
    end
end
% Current number of endmembers
p_current = p_init;

%% Noise estimation and noise tolerance for searching the actual p value
[w, Rw] = estNoise(X);
tolerance = mean(diag(Rw));

%% PCA projection

% Data average spectra
r_m = mean(X,2);
% De-meaning
R_m = repmat(r_m,[1 N]); % mean of each band
R_o = X - R_m;           % data with zero-mean
% PCA
[Ud,~,~] = svds(R_o*R_o'/N,p-1);  % computes the p-projection matrix (up to p_end)
% Projection
X_p0 =  Ud(:,1:p_current-1)' * R_o; % project thezeros mean data onto p-subspace
% Add constant row with the maximum norm component
c = max(sum(X_p0.^2,1))^0.5;
X_p = [X_p0 ; c*ones(1,N)];

%% Random seed

% random seed selection
if isempty(seed)
    seed = ceil(N*rand(1,p_current));
end
% check if it is well conditioned
E = X_p(:,seed);
M = E.'*E;
while (rcond(M) < 5*10^(-15))
    seed = ceil(N*rand(1,p_current));
    E = X_p(:,seed);
    M = E.'*E;
end

%% Initializations

% Initial energy
Abundances_min_Energy = M\E.'*X_p; % Pseudoinverse
[~, j_sorted_list] = sort(min(Abundances_min_Energy));
min_Energia = Energy(Abundances_min_Energy);

% Endmembers indexes
E_index = seed;

%% Look for the best solution for current p value
stop_condition = false;
while ~stop_condition

    % Energy index
    j_index = 1;
    
    % Iterative solution for current p value
    % Check for replacements until no improvment could be found for
    % iterations
    while j_index <= iterations
        % Initialize replacement indexes [endmember,pixel]
        replacement = [0 0]; 
        % Replacement pixel
        j = j_sorted_list(j_index);
        % Test all endmembers to find the best replacement
        for i=1:p_current
            % Replace i-th endmember by  j-th pixel
            saved_E_index = E_index(i); % save replaced endmember index
            E_index(i) = j;
            % Endmembers
            E = X_p(:,E_index);
            % Covariance matrix
            M = E.'*E;
            if (rcond(M) >= 10^(-15)) % Well conditioned?
                % Abundances estimation
                Abundances = M\E.'*X_p;
                % Energy
                Energy_replacement = Energy(Abundances);
                % Improve?
                if Energy_replacement < min_Energia
                    % Save the currently best replacement
                    replacement = [i j]; 
                    min_Energia = Energy_replacement;
                    Abundances_min_Energy = Abundances;
                end
            end
            % Undo replacement
            E_index(i) = saved_E_index;
        end
        % there is an improving replacement?
        if replacement(1) % Yes
            % Replace the i-th endmember by the j-th pixel
            E_index(replacement(1)) = replacement(2);
            % Sort the new energy values
            [~, j_sorted_list] = sort(min(Abundances_min_Energy));
            % Re-initialize the energy index
            j_index = 1;
        else % No
            % Check next energy value
            j_index = j_index + 1; 
        end
    end
    
    % No more replacements
    % Check stop condition
    if p_current == p
        stop_condition = true;
    elseif p_search
        % Endmembers
        Endmembers = X(:,E_index);
        % Unconstrained abundances
        Abundances_min_Energy = pinv(Endmembers)*X;
        % Reconstruct data
        X_est = Endmembers*Abundances_min_Energy;
        % Error power
        error_power = mean((X_est(:) - X(:)).^2);
        % Endmembers noise power
        noise_endmembers = w(:,E_index)*Abundances_min_Energy;
        E_noise_power = mean(mean(noise_endmembers.^2));
        % Check condition
        if error_power <= tolerance + E_noise_power
            stop_condition = true;
        end
    end
    
    % If stop condition is not achieved
    % Increase p
    if ~stop_condition
        p_current = p_current + 1;
        X_p0 = Ud(:,1:p_current-1)' * R_o;
        X_p = [X_p0 ; c*ones(1,N)];
        E_index = [E_index j_sorted_list(1)]; 
        Abundances_min_Energy = pinv(X_p(:,E_index))*X_p;
        [~, j_sorted_list] = sort(min(Abundances_min_Energy));
        min_Energia = Energy(Abundances_min_Energy);
    end
    
end   

%% OUTPUT

% Endmembers in the original domain
Ud = Ud(:,1:p_current-1);
X_p =  Ud' * R_o(:,E_index); 
Rp =  Ud * X_p + R_m(:,E_index); 
E = Rp;

end

function J = Energy(Abundances)
    min_Abundances = min(Abundances);
    J = abs(sum(min_Abundances(min_Abundances < 0)));
end